/***************************************************************************//**
* \file CapSense_1_Adc.c
* \version 3.0
*
* \brief
*   This file provides implementation for the ADC module of the CapSense_1
*   Component.
*
* \see CapSense_1 v3.0 Datasheet
*
*//*****************************************************************************
* Copyright (2016-2017), Cypress Semiconductor Corporation.
********************************************************************************
* This software is owned by Cypress Semiconductor Corporation (Cypress) and is
* protected by and subject to worldwide patent protection (United States and
* foreign), United States copyright laws and international treaty provisions.
* Cypress hereby grants to licensee a personal, non-exclusive, non-transferable
* license to copy, use, modify, create derivative works of, and compile the
* Cypress Source Code and derivative works for the sole purpose of creating
* custom software in support of licensee product to be used only in conjunction
* with a Cypress integrated circuit as specified in the applicable agreement.
* Any reproduction, modification, translation, compilation, or representation of
* this software except as specified above is prohibited without the express
* written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the
* materials described herein. Cypress does not assume any liability arising out
* of the application or use of any product or circuit described herein. Cypress
* does not authorize its products for use as critical components in life-support
* systems where a malfunction or failure may reasonably be expected to result in
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of
* such use and in doing so indemnifies Cypress against all charges. Use may be
* limited by and subject to the applicable Cypress software license agreement.
*******************************************************************************/
#include "syslib/cy_syslib.h"
#include <stddef.h>
#include "gpio/cy_gpio.h"
#include "CapSense_1_Configuration.h"
#if (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN)
    #include "syspm/cy_syspm.h"
    #include "cyfitter_sysint_cfg.h"
#endif  /* (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN) */
#include "CapSense_1_ModClk.h"
#include "CapSense_1_Structure.h"
#include "CapSense_1_Adc.h"
#include "CapSense_1_Sensing.h"
#if (CapSense_1_ENABLE == CapSense_1_SELF_TEST_EN)
    #include "CapSense_1_SelfTest.h"
#endif

#if (CapSense_1_ADC_EN)

static uint8 CapSense_1_initVar;
uint16 CapSense_1_adcVrefMv = CapSense_1_ADC_VREF_MV;


/*******************************************************************************
* Module local function declarations
*******************************************************************************/
/**
* \cond SECTION_ADC_INTERNAL
* \addtogroup group_adc_internal
* \{
*/

#if (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN)
    static void CapSense_1_SsIsrInitialize(cy_israddress address);
#endif  /* (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN) */

/** \}
* \endcond */

/*******************************************************************************
* Local definition
*******************************************************************************/
#define CapSense_1_AdcINIT_DONE                    (1u)
#define CapSense_1_AdcINIT_NEEDED                  (0u)
#define CapSense_1_AdcCAL_WATCHDOG_CYCLES_NUM      (0x0000FFFFLu)

#if (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN)
    /*******************************************************************************
    * Function Name: CapSense_1_Start
    ****************************************************************************//**
    * \if SECTION_STANDALONE_ADC
    * \brief
    *   Configures the hardware and performs calibration.
    *
    * \details
    *   Configures the hardware and performs calibration.
    * \endif
    *
    *******************************************************************************/
    void CapSense_1_Start(void)
    {
        CapSense_1_ClearAdcChannels();

        if (CapSense_1_AdcINIT_DONE != CapSense_1_initVar)
        {
            CapSense_1_AdcDsInitialize();
        }
        CapSense_1_ConfigAdcResources();
        (void) CapSense_1_AdcCalibrate();
        while(CapSense_1_AdcIsBusy() != CapSense_1_AdcSTATUS_IDLE)
        {
        }
    }

    /*******************************************************************************
    * Function Name: CapSense_1_Sleep
    ****************************************************************************//**
    * \if SECTION_STANDALONE_ADC
    * \brief
    *  Prepares the Component for deep sleep.
    *
    * \details
    *  Currently this function is empty and exists as a place for future updates,
    *  this function will be used to prepare the Component to enter deep sleep.
    * \endif
    *
    *******************************************************************************/
    void CapSense_1_Sleep(void)
    {
    }


    /*******************************************************************************
    * Function Name: CapSense_1_Wakeup
    ****************************************************************************//**
    * \if SECTION_STANDALONE_ADC
    * \brief
    *  This function resumes the Component after sleep.
    *
    * \details
    *  Currently this function is empty and exists as a place for future updates,
    *  this function will be used to resume the Component after exiting deep sleep.
    * \endif
    *
    *******************************************************************************/
    void CapSense_1_Wakeup(void)
    {
    }
#endif  /* (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN) */

/*******************************************************************************
* Function Name: CapSense_1_SetNonDedicatedAdcChannel
****************************************************************************//**
*
* \brief
*   Sets the non dedicated channel to the given state.
*
* \details
*   Connects/disconnects the pin and the analog muxbus B. Sets the drive mode
*   of the pin as well.
*
* \param chId  The ID of the non dedicated channel to be set.
* \param state The state in which the channel is to be put:
*         - (0) CapSense_1_AdcCHAN_DISCONNECT
*         - (1) CapSense_1_AdcCHAN_CONNECT
*
*******************************************************************************/
void CapSense_1_SetNonDedicatedAdcChannel(uint8 chId, uint32 state)
{
    CapSense_1_FLASH_IO_STRUCT const *ptr2adcIO;
    uint32  interruptState;

    ptr2adcIO = &CapSense_1_adcIoList[chId];
    interruptState = Cy_SysLib_EnterCriticalSection();
    switch (state)
    {
    case CapSense_1_AdcCHAN_CONNECT:
        /* Connect AMuxBusB to the selected port */
        Cy_GPIO_SetHSIOM((GPIO_PRT_Type*)ptr2adcIO->pcPtr, (uint32)ptr2adcIO->pinNumber, (en_hsiom_sel_t)HSIOM_SEL_AMUXB);
        /* Update port configuration register (drive mode) to HiZ Analog */
        Cy_GPIO_SetDrivemode((GPIO_PRT_Type*)ptr2adcIO->pcPtr, (uint32)ptr2adcIO->pinNumber, CY_GPIO_DM_ANALOG);
        break;

    /* Disconnection is a safe default state. Fall-through is intentional. */
    case CapSense_1_AdcCHAN_DISCONNECT:
    default:
        /* Disconnec AMuxBusB from the selected port */
        Cy_GPIO_SetHSIOM((GPIO_PRT_Type*)ptr2adcIO->pcPtr, (uint32)ptr2adcIO->pinNumber, (en_hsiom_sel_t)HSIOM_SEL_GPIO);
        break;
    }
    Cy_SysLib_ExitCriticalSection(interruptState);
}


/*******************************************************************************
* Function Name: CapSense_1_SetAdcChannel
****************************************************************************//**
*
* \brief
*   Sets the given channel to the given state.
*
* \details
*   Connects/disconnects the pin and the analog muxbus B.  Sets the drive mode
*   of the pin as well.
*
* \param chId  The ID of the channel to be set.
* \param state The state in which the channel is to be put:
*         - (0) CapSense_1_AdcCHAN_DISCONNECT
*         - (1) CapSense_1_AdcCHAN_CONNECT
*
*******************************************************************************/
void CapSense_1_SetAdcChannel(uint8 chId, uint32 state)
{
    #if (0u != CapSense_1_ADC_AMUXB_INPUT_EN)
        #if (CapSense_1_ADC_MIN_CHANNELS < CapSense_1_ADC_TOTAL_CHANNELS)
            if(CapSense_1_ADC_SELECT_AMUXB_CH != chId)
            {
                CapSense_1_SetNonDedicatedAdcChannel(chId, state);
            }
        #endif /* (CapSense_1_ADC_MIN_CHANNELS < CapSense_1_ADC_TOTAL_CHANNELS) */
    #else
        CapSense_1_SetNonDedicatedAdcChannel(chId, state);
    #endif /* (0u != CapSense_1_ADC_AMUXB_INPUT_EN) */
}


/*******************************************************************************
* Function Name: CapSense_1_ConfigAdcResources
****************************************************************************//**
*
* \brief
*   Configures the CSD block to be used as an ADC.
*
* \details
*   Configures the IDACB, internal switches, REFGEN, HSCOMP, enables CSD
*   block interrupt and set interrupt vector to ADC sensing method.
*
*******************************************************************************/
void CapSense_1_ConfigAdcResources(void)
{
    uint32 interruptState;
    uint32 newRegValue;

    /* Configure clocks */
    CapSense_1_ModClk_SetDivider(CapSense_1_ADC_MODCLK_DIV_DEFAULT - 1u);
    CY_SET_REG32(CapSense_1_CSD_SENSE_PERIOD_PTR, (CapSense_1_AdcSENSE_DIV_DEFAULT - 1u));

    /* Configure the IDAC */
    CY_SET_REG32(CapSense_1_CSD_CONFIG_PTR, CapSense_1_AdcCONFIG_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_IDACB_PTR, CapSense_1_AdcIDACB_CONFIG | CapSense_1_dsRam.adcIdac);

    /* Configure AZ Time */
    CY_SET_REG32(CapSense_1_CSD_SEQ_TIME_PTR, (CapSense_1_AdcSEQ_TIME_DEFAUL - 1u));

    CY_SET_REG32(CapSense_1_CSD_CSDCMP_PTR, 0ul);
    CY_SET_REG32(CapSense_1_CSD_SW_DSI_SEL_PTR, 0ul);

    CY_SET_REG32(CapSense_1_CSD_SENSE_DUTY_PTR, 0ul);
    CY_SET_REG32(CapSense_1_CSD_SEQ_INIT_CNT_PTR, 1ul);
    CY_SET_REG32(CapSense_1_CSD_SEQ_NORM_CNT_PTR, 2ul);

    /* Configure the block-level routing */
    CY_SET_REG32(CapSense_1_CSD_SW_HS_P_SEL_PTR, CapSense_1_AdcSW_HSP_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_HS_N_SEL_PTR, CapSense_1_AdcSW_HSN_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_SHIELD_SEL_PTR, CapSense_1_AdcSW_SHIELD_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_CMP_P_SEL_PTR, CapSense_1_AdcSW_CMPP_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_CMP_N_SEL_PTR, CapSense_1_AdcSW_CMPN_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_FW_MOD_SEL_PTR, CapSense_1_AdcSW_FWMOD_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_FW_TANK_SEL_PTR, CapSense_1_AdcSW_FWTANK_DEFAULT);
    CY_SET_REG32(CapSense_1_CSD_SW_REFGEN_SEL_PTR, CapSense_1_AdcSW_REFGEN_DEFAULT);

    interruptState = Cy_SysLib_EnterCriticalSection();
    newRegValue = CY_GET_REG32(CapSense_1_CSD_SW_BYP_SEL_PTR);
    newRegValue |= CapSense_1_AdcSW_BYP_DEFAULT;
    CY_SET_REG32(CapSense_1_CSD_SW_BYP_SEL_PTR, newRegValue);
    Cy_SysLib_ExitCriticalSection(interruptState);

    /* Config RefGen */
    CY_SET_REG32(CapSense_1_CSD_REFGEN_PTR, CapSense_1_AdcREFGEN_NORM);
    CY_SET_REG32(CapSense_1_CSD_SW_AMUXBUF_SEL_PTR,
            CapSense_1_AdcSW_AMUBUF_NORM);

    /* Configure HSCOMP */
    CY_SET_REG32(CapSense_1_CSD_HSCMP_PTR, CapSense_1_AdcHSCMP_AZ_DEFAULT);

    /* Enable CSD block interrupt and set interrupt vector to ADC sensing method */
    CapSense_1_SsIsrInitialize(&CapSense_1_AdcIntrHandler);

    /* Component is initialized */
    CapSense_1_initVar = CapSense_1_AdcINIT_DONE;
}


/*******************************************************************************
* Function Name: CapSense_1_StartAdcFSM
****************************************************************************//**
*
* \brief
*   Starts the CSD state machine with correct parameters to initialize an ADC
*   conversion.
*
* \details
*   Starts the CSD state machine with correct parameters to initialize an ADC
*   conversion.
*
* \param measureMode The FSM mode:
*        - (0) CapSense_1_AdcMEASMODE_OFF
*        - (1) CapSense_1_AdcMEASMODE_VREF
*        - (2) CapSense_1_AdcMEASMODE_VREFBY2
*        - (3) CapSense_1_AdcMEASMODE_VIN
*
*******************************************************************************/
void CapSense_1_StartAdcFSM(uint32 measureMode)
{
    uint32 tmpStartVal;

    /* Set the mode and acquisition time */
    CY_SET_REG32(CapSense_1_CSD_ADC_CTL_PTR, (measureMode | (CapSense_1_AdcACQUISITION_BASE - 1u)));

    if(CapSense_1_AdcMEASMODE_VREF == measureMode)
    {
        tmpStartVal =
            CapSense_1_AdcFSMSETTING_AZSKIP_DEFAULT    |
            CapSense_1_AdcFSMSETTING_DSIIGNORE   |
            CapSense_1_AdcFSMSETTING_NOABORT     |
            CapSense_1_AdcFSMSETTING_SEQMODE     |
            CapSense_1_AdcFSMSETTING_START;
    }
    else if (CapSense_1_AdcMEASMODE_OFF == measureMode)
    {
        tmpStartVal = CapSense_1_AdcFSMSETTING_ABORT;
    }
    /* This setting is used for both MEASMODE_VREFBY2 and MEASMODE_VIN */
    else
    {
        tmpStartVal =
            CapSense_1_AdcFSMSETTING_AZSKIP_DEFAULT    |
            CapSense_1_AdcFSMSETTING_DSIIGNORE   |
            CapSense_1_AdcFSMSETTING_NOABORT     |
            CapSense_1_AdcFSMSETTING_SEQMODE     |
            CapSense_1_AdcFSMSETTING_START;
    }

    /* Enable HSComp */
    CY_SET_REG32(CapSense_1_CSD_SEQ_START_PTR, tmpStartVal);
}

/*******************************************************************************
* Function Name: CapSense_1_AdcCaptureResources
****************************************************************************//**
*
* \brief
*   Releases CSD resources from sensing mode, and sets it into ADC mode.
*
* \details
*   Releases CSD resources from sensing mode, and sets it into ADC mode.
*
* \return     The function returns cy_status of its operation.
*   - CY_RET_LOCKED  - The sensing hardware is in-use and could not be released.
*   - CY_RET_SUCCESS - Block is configured for ADC use.
*
*******************************************************************************/
cy_status CapSense_1_AdcCaptureResources(void)
{
    cy_status tmpStatus = CY_RET_SUCCESS;

    #if !(CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN)
        tmpStatus = CapSense_1_SsReleaseResources();

        if (CapSense_1_AdcINIT_NEEDED == CapSense_1_initVar)
        {
            if(CY_RET_SUCCESS == tmpStatus)
            {
                CapSense_1_ConfigAdcResources();
            }
            else
            {
                tmpStatus = CY_RET_LOCKED;
            }
        }
    #else
        if (CapSense_1_AdcINIT_NEEDED == CapSense_1_initVar)
        {
            CapSense_1_ConfigAdcResources();
        }
    #endif  /* !(CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN) */

    return tmpStatus;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcReleaseResources
****************************************************************************//**
*
* \brief
*   Releases CSD resources from ADC mode.
*
* \details
*   Releases CSD resources from ADC mode.
*
* \return     The function returns cy_status of its operation.
*   CY_RET_SUCCESS   Block resources no longer in use.
*
*******************************************************************************/
cy_status CapSense_1_AdcReleaseResources(void)
{
    uint32 interruptState;
    uint32 newRegValue;

    if (CapSense_1_AdcINIT_DONE == CapSense_1_initVar)
    {
        /* If the FSM is running, shut it down. */
        if(CapSense_1_AdcSTATUS_IDLE != (CapSense_1_dsRam.adcStatus
                            & CapSense_1_AdcSTATUS_FSM_MASK))
        {
            CapSense_1_StartAdcFSM(CapSense_1_AdcMEASMODE_OFF);
            CapSense_1_SetAdcChannel((CapSense_1_dsRam.adcStatus
                            & (uint8)(CapSense_1_AdcSTATUS_LASTCHAN_MASK)),
                            CapSense_1_AdcCHAN_DISCONNECT);
            CapSense_1_dsRam.adcStatus = CapSense_1_AdcSTATUS_IDLE;
        }

        /* Disable the subblocks. */
        CY_SET_REG32(CapSense_1_CSD_IDACB_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_REFGEN_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_AMBUF_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_HSCMP_PTR, 0u);

        /* Reset the block-level routing */
        CY_SET_REG32(CapSense_1_CSD_SW_HS_P_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_HS_N_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_SHIELD_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_CMP_P_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_CMP_N_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_FW_MOD_SEL_PTR, 0u);
        CY_SET_REG32(CapSense_1_CSD_SW_FW_TANK_SEL_PTR, 0u);

        interruptState = Cy_SysLib_EnterCriticalSection();
        newRegValue = CY_GET_REG32(CapSense_1_CSD_SW_BYP_SEL_PTR);
        newRegValue &= (uint32)(~CapSense_1_AdcSW_BYP_DEFAULT);
        CY_SET_REG32(CapSense_1_CSD_SW_BYP_SEL_PTR, newRegValue);
        Cy_SysLib_ExitCriticalSection(interruptState);

         /* Disconnect all ADC channels */
        CapSense_1_ClearAdcChannels();

        CapSense_1_initVar = CapSense_1_AdcINIT_NEEDED;
    }

    return CY_RET_SUCCESS;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcStartConvert
****************************************************************************//**
*
* \brief
*  Initializes the hardware and initiates an analog-to-digital conversion on the
*  selected input channel.
*
* \details
*  Initializes the hardware and initiates an analog-to-digital conversion on the
*  selected input channel. This API only initiates a conversion and does not
*  wait for the conversion to be completed, therefore the
*  CapSense_1_AdcIsBusy() API must be used to check the
*  status and ensure that the conversion is complete prior to reading the result,
*  starting a new conversion with the same or a different channel or reconfiguring
*  the hardware for different functionality.
*
* \param chId
*  The ID of the channel to be converted.
*
* \return
*  The function returns cystatus of its operation.
*    - CY_RET_SUCCESS - A conversion has started.
*    - CY_RET_LOCKED - The hardware is already in-use by a previously initialized
*      conversion or other functionality. No new conversion is started by this API.
*    - CY_RET_BAD_PARAM - An invalid channel Id. No conversion is started.
*
*******************************************************************************/
cy_status CapSense_1_AdcStartConvert(uint8 chId)
{
    uint32 tmpStatus = CY_RET_SUCCESS;

    /* If non-standalone ADC, try to Capture resources */
    #if (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN)
        tmpStatus = CapSense_1_AdcCaptureResources();

        if (CY_RET_SUCCESS == tmpStatus)
        {
    #else /* Otherwise, configure resources if needed. */
        if (CapSense_1_AdcINIT_NEEDED == CapSense_1_initVar)
        {
            CapSense_1_ConfigAdcResources();
        }
    #endif /* (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN) */

    if(chId >= CapSense_1_ADC_TOTAL_CHANNELS)
    {
        tmpStatus = CY_RET_BAD_PARAM;
    }
    
    if (CY_RET_SUCCESS == tmpStatus)
    {
        if(CapSense_1_AdcSTATUS_IDLE != (CapSense_1_dsRam.adcStatus & CapSense_1_AdcSTATUS_FSM_MASK))
        {
            tmpStatus = CY_RET_LOCKED;
        }

        if(CY_RET_SUCCESS == tmpStatus)
        {
            #if(CapSense_1_ADC_ANALOG_STARTUP_DELAY_US > 0uL)
                Cy_SysLib_DelayUs(CapSense_1_ADC_ANALOG_STARTUP_DELAY_US);
            #endif /* (CapSense_1_ADC_ANALOG_STARTUP_DELAY_US > 0uL) */

            /* Set Component Status */
            CapSense_1_dsRam.adcStatus = (CapSense_1_AdcSTATUS_CONVERTING | chId);

            /* Configure a desired channel if needed */
            if (chId != CapSense_1_dsRam.adcActiveCh)
            {
                if (CapSense_1_AdcNO_CHANNEL != CapSense_1_dsRam.adcActiveCh)
                {
                    /* Disconnect existing channel */
                    CapSense_1_SetAdcChannel(CapSense_1_dsRam.adcActiveCh, CapSense_1_AdcCHAN_DISCONNECT);
                }
                /* Connect desired input */
                CapSense_1_SetAdcChannel(chId, CapSense_1_AdcCHAN_CONNECT);
                CapSense_1_dsRam.adcActiveCh = chId;
            }

            /* Un-mask ADC_RES interrupt (enable interrupt) */
            CY_SET_REG32(CapSense_1_CSD_INTR_MASK_PTR, CapSense_1_CSD_INTR_MASK_ADC_RES_MSK);
            CapSense_1_StartAdcFSM(CapSense_1_AdcMEASMODE_VIN);
        }
    }
    
    #if (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN)
        }
    #endif /* (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN) */
    
    return tmpStatus;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcIsBusy
****************************************************************************//**
*
* \brief
*   The function returns the status of the ADC's operation.
*
* \details
*   The function returns the status of the ADC's operation. A new conversion or
*   calibration must not be started unless the ADC is in the IDLE state.
*
* \return
*  The function returns the status of the ADC's operation.
*    - CapSense_1_AdcSTATUS_IDLE - The ADC is not busy,
*       a new conversion can be initiated.
*    - CapSense_1_AdcSTATUS_CONVERTING - A previously
*       initiated conversion is in progress.
*    - CapSense_1_AdcSTATUS_CALIBPH1 - The ADC is in the
*      first phase (of 3) of calibration.
*    - CapSense_1_AdcSTATUS_CALIBPH2 - The ADC is in the
*      second phase (of 3) of calibration.
*    - CapSense_1_AdcSTATUS_CALIBPH3 - The ADC is in the
*      third phase (of 3) of calibration.
*    - CapSense_1_AdcSTATUS_OVERFLOW - The most recent
*      measurement caused an overflow. The root cause of the overflow may be
*      the previous calibration values being invalid or the VDDA setting in cydwr
*      and hardware do not match. Perform re-calibration or set the
*      appropriate VDDA value in cydwr to avoid this error condition.
*
*******************************************************************************/
uint8 CapSense_1_AdcIsBusy(void)
{
    uint8 tmpStatus;

    if (0u != (CapSense_1_CSD_ADC_RES_REG & CapSense_1_AdcADC_RES_OVERFLOW_MASK))
    {
        tmpStatus = CapSense_1_AdcSTATUS_OVERFLOW;
    }
    else
    {
        tmpStatus = (uint8)((*(volatile uint8 *)&CapSense_1_dsRam.adcStatus) & CapSense_1_AdcSTATUS_FSM_MASK);
    }

    return tmpStatus;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcReadResult_mVolts
****************************************************************************//**
*
* \brief
*  This is a blocking API. It initiates a conversion, waits for completion and
*  returns the result.
*
* \details
*  This is a blocking API. Internally, it starts a conversion using
*  CapSense_1_AdcStartConvert(), checks the status using
*  CapSense_1_AdcIsBusy(), waits until the conversion is
*  completed and returns the result.
*
* \param chId
*  The ID of the channel to be measured
*
* \return
*  The function returns voltage in millivolts or
*  CapSense_1_AdcVALUE_BAD_RESULT if:
*   - chId is invalid
*   - The ADC conversion is not started
*   - The ADC conversion watch-dog triggered.
*******************************************************************************/
uint16 CapSense_1_AdcReadResult_mVolts(uint8 chId)
{
    cy_status convertStatus;
    uint16 tmpValue;
    uint32 watchdogAdcCounter;

    convertStatus = CapSense_1_AdcStartConvert(chId);
    if (CY_RET_SUCCESS == convertStatus)
    {
        /* Initialize Watchdog Counter with time interval which is enough to ADC conversion is completed */
        watchdogAdcCounter = CapSense_1_AdcCAL_WATCHDOG_CYCLES_NUM;
        while ((CapSense_1_AdcIsBusy() != CapSense_1_AdcSTATUS_IDLE) &&
               (0u != watchdogAdcCounter))
        {
            /* Wait until conversion complete and decrement Watchdog Counter to prevent unending loop */
            watchdogAdcCounter--;
        }
        if (0u != watchdogAdcCounter)
        {
            tmpValue = CapSense_1_AdcGetResult_mVolts(chId);
        }
        else
        {
            tmpValue = (uint16) CapSense_1_AdcVALUE_BAD_RESULT;
        }
    }
    else
    {
        tmpValue = (uint16) CapSense_1_AdcVALUE_BAD_RESULT;
    }

    return tmpValue;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcGetResult_mVolts
****************************************************************************//**
*
* \brief
*  This API does not perform an ADC conversion and returns the last valid result
*  for the specified channel.
*
* \details
*  Returns the last valid result from the data structure for the specified channel.
*  This function can be used to read a previous result of any channel even if the
*  ADC is busy or a conversion is in progress. However, it is highly recommended
*  not to use this function with a channel that is in an active conversion.
*
* \param chId
*  The ID of the channel to be measured
*
* \return
*  The function returns a voltage in millivolts or
*  CapSense_1_AdcVALUE_BAD_CHAN_ID if chId is invalid.
*
*******************************************************************************/
uint16 CapSense_1_AdcGetResult_mVolts(uint8 chId)
{
    uint32 tmpRetVal = CapSense_1_AdcVALUE_BAD_CHAN_ID;

    if(chId < CapSense_1_ADC_TOTAL_CHANNELS)
    {
        tmpRetVal = CapSense_1_dsRam.adcResult[chId];
    }
    return (uint16)tmpRetVal;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcCalibrate
****************************************************************************//**
*
* \brief
*  Performs calibration of the ADC module.
*
* \details
*  Performs calibration for the ADC to identify the appropriate hardware configuration
*  to produce accurate results. It is recommended to run the calibration
*  periodically (for example every 10 seconds) for accuracy and compensations.
*
* \return
*  The function returns cystatus of its operation.
*    - CY_RET_SUCCESS - The block is configured for the ADC use.
*    - CY_RET_LOCKED - The hardware is already in-use by a previously initialized
*      conversion or other functionality. No new conversion is started by this API.
*
*******************************************************************************/
cy_status CapSense_1_AdcCalibrate(void)
{
    uint32 tmpStatus;
    uint32 watchdogAdcCounter;
    uint32 tmpVrefCal;

    tmpStatus = CapSense_1_AdcCaptureResources();

    if(CapSense_1_AdcSTATUS_IDLE != (CapSense_1_dsRam.adcStatus
        & CapSense_1_AdcSTATUS_FSM_MASK))
    {
        tmpStatus = CY_RET_LOCKED;
    }

    if (CY_RET_SUCCESS == tmpStatus)
    {
        #if(CapSense_1_ADC_ANALOG_STARTUP_DELAY_US > 0uL)
            Cy_SysLib_DelayUs(CapSense_1_ADC_ANALOG_STARTUP_DELAY_US);
        #endif /* (CapSense_1_ADC_ANALOG_STARTUP_DELAY_US > 0uL) */

        /* Disconnect a channel if connected */
        if (CapSense_1_AdcNO_CHANNEL != CapSense_1_dsRam.adcActiveCh)
        {
            CapSense_1_SetAdcChannel(CapSense_1_dsRam.adcActiveCh, CapSense_1_AdcCHAN_DISCONNECT);
            CapSense_1_dsRam.adcActiveCh = CapSense_1_AdcNO_CHANNEL;
        }
        
        /* Get Vref trim-value */
        tmpVrefCal = (CapSense_1_AdcVREF_TRIM_REG & CapSense_1_AdcVREF_TRIM_MSK) >>
            CapSense_1_AdcVREF_TRIM_POS;

        /* Update nominal Vref to real Vref */
        tmpVrefCal *= CapSense_1_ADC_VREF_MV;
        tmpVrefCal /= CapSense_1_AdcVREF_CALIB_USED;

        CapSense_1_adcVrefMv = (uint16)tmpVrefCal;

        /* Calculate deviation of trim register */
        if (tmpVrefCal > CapSense_1_ADC_VREF_MV)
        {
            tmpVrefCal = tmpVrefCal - CapSense_1_ADC_VREF_MV;
        }
        else
        {
            tmpVrefCal = CapSense_1_ADC_VREF_MV - tmpVrefCal;
        }
        tmpVrefCal = (tmpVrefCal * CapSense_1_PERCENTAGE_100) / CapSense_1_ADC_VREF_MV;

        /* Use nominal Vref if trim-value is not within the allowed range */
        if (CapSense_1_AdcVREF_TRIM_MAX_DEVIATION < tmpVrefCal)
        {
            CapSense_1_adcVrefMv = CapSense_1_ADC_VREF_MV;
        }

        CapSense_1_dsRam.adcIdac = (uint8)CapSense_1_ADC_IDAC_DEFAULT;
        CY_SET_REG32(CapSense_1_CSD_IDACB_PTR, CapSense_1_AdcIDACB_CONFIG |
                                                     CapSense_1_dsRam.adcIdac);

        CapSense_1_dsRam.adcStatus = (CapSense_1_AdcSTATUS_CALIBPH1);

        /* Un-mask ADC_RES interrupt (enable interrupt) */
        CY_SET_REG32(CapSense_1_CSD_INTR_MASK_PTR, CapSense_1_CSD_INTR_MASK_ADC_RES_MSK);
        CapSense_1_StartAdcFSM(CapSense_1_AdcMEASMODE_VREF);

        #if (CapSense_1_ENABLE == CapSense_1_SELF_TEST_EN)
            /* Global CRC update */
            #if (CapSense_1_ENABLE ==CapSense_1_TST_GLOBAL_CRC_EN)
                CapSense_1_DsUpdateGlobalCrc();
            #endif /* (CapSense_1_ENABLE == CapSense_1_TST_GLOBAL_CRC_EN) */
        #endif /* (CapSense_1_ENABLE == CapSense_1_SELF_TEST_EN) */

        /* Initialize Watchdog Counter with time interval which is enough to ADC calibration is completed */
        watchdogAdcCounter = CapSense_1_AdcCAL_WATCHDOG_CYCLES_NUM;
        while (((*(volatile uint8 *)&CapSense_1_dsRam.adcStatus & CapSense_1_AdcSTATUS_FSM_MASK)
                != 0u) &&  (0u != watchdogAdcCounter))
        {
            /* Wait until scan complete and decrement Watchdog Counter to prevent unending loop */
            watchdogAdcCounter--;
        }
    }

    return tmpStatus;
}


/*******************************************************************************
* Function Name: CapSense_1_AdcInitialize
****************************************************************************//**
*
* \brief
*   Configures the hardware to ADC mode and begins a calibration.
*
* \details
*   Configures the hardware to ADC mode and begins a calibration.
*
*******************************************************************************/
void CapSense_1_AdcInitialize(void)
{
    CapSense_1_ConfigAdcResources();
    (void)CapSense_1_AdcCalibrate();
}


/*******************************************************************************
* Function Name: CapSense_1_Stop
****************************************************************************//**
*
* \brief
*   Disables the hardware sub-blocks that are in use while in the ADC mode,
*   and frees the routing.
*
* \details
*   This function stops the Component operation. No ADC conversion can be
*   initiated when the Component is stopped. Once stopped, the hardware block
*   may be reconfigured by the application program for any other special usage.
*   The ADC operation can be resumed by calling the
*   CapSense_1_Resume() function or the Component can
*   be reset by calling the CapSense_1_Start() function.
*   This function is called when no ADC conversion is in progress.
*
*******************************************************************************/
void CapSense_1_Stop(void)
{
    #if (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN)
        (void)CapSense_1_AdcReleaseResources();
    #endif /* (CapSense_1_ENABLE != CapSense_1_ADC_STANDALONE_EN) */
    CapSense_1_initVar = CapSense_1_AdcINIT_NEEDED;
}


/*******************************************************************************
* Function Name: CapSense_1_Resume
****************************************************************************//**
*
* \brief
*   Resumes the ADC operation after a stop call.
*
* \details
*   Resumes the ADC operation if the operation is stopped
*   previously by the CapSense_1_Stop() API.
*
*******************************************************************************/
void CapSense_1_Resume(void)
{
    CapSense_1_AdcInitialize();
}


/*******************************************************************************
* Function Name: CapSense_1_ClearAdcChannels
****************************************************************************//**
*
* \brief
*  Resets all the ADC channels to disconnected state.
*
* \details
*   The function goes through all the ADC channels and disconnects the pin
*   and the analog muxbus B.  Sets the drive mode of the pin as well.
*
*******************************************************************************/
void CapSense_1_ClearAdcChannels(void)
{
    uint32 chId;

    for (chId = 0u; chId < CapSense_1_ADC_TOTAL_CHANNELS; chId++)
    {
        CapSense_1_SetAdcChannel((uint8)chId, CapSense_1_AdcCHAN_DISCONNECT);
    }
    CapSense_1_dsRam.adcActiveCh = CapSense_1_AdcNO_CHANNEL;
}

#if (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN)
/*******************************************************************************
* Function Name: CapSense_1_SsIsrInitialize
****************************************************************************//**
*
* \brief
*   Enables and initializes for the function pointer for a callback for the ISR.
*
* \details
*   The  "address" is a special type cy_israddress defined by syslib. This function
*   is used by Component APIs and should not be used by an application program for
*   proper working of the Component.
*
* \param  address The address of the function to be called when interrupt is fired.
*
*******************************************************************************/
static void CapSense_1_SsIsrInitialize(cy_israddress address)
{
    /* Disable interrupt */
    #if defined(CapSense_1_ISR__INTC_ASSIGNED)
        NVIC_DisableIRQ(CapSense_1_ISR_cfg.intrSrc);
    #endif

    /* Configure interrupt with priority and vector */
    #if defined(CapSense_1_ISR__INTC_ASSIGNED)
        (void)Cy_SysInt_Init(&CapSense_1_ISR_cfg, address);
    #endif

    /* Enable interrupt */
    #if defined(CapSense_1_ISR__INTC_ASSIGNED)
        NVIC_EnableIRQ(CapSense_1_ISR_cfg.intrSrc);
    #endif
}


/*******************************************************************************
* Function Name: CapSense_1_DeepSleepCallback
****************************************************************************//**
*
* Handles Active to Deep Sleep power mode transition for the CapSense_1
* Component.
*
* Calling this function directly from the application layer is not recommended.
* Instead, Cy_SysPm_CpuEnterDeepSleep() should be used for the Active to Deep Sleep
* power mode transition of the device.
*
* For proper operation of the CapSense_1 Component during the Active to
* Deep Sleep mode transition, a callback to this function should be registered
* using the Cy_SysPm_RegisterCallback() function with CY_SYSPM_DEEPSLEEP
* type. After the callback is registered, this function is called by the
* Cy_SysPm_CpuEnterDeepSleep() function to prepare the Component to the device
* power mode transition.
*
* When this function is called with CY_SYSPM_CHECK_READY as input, this
* function returns CY_SYSPM_SUCCESS if no scanning is in progress. Otherwise
* CY_SYSPM_FAIL is returned. If CY_SYSPM_FAIL status is returned, a device
* cannot change the power mode without completing the current scan as
* a transition to Deep Sleep during the scan can disrupt the Component
* operation.
*
* \param callbackParams
* Refer to the description of the cy_stc_syspm_callback_params_t type in the
* Peripheral Driver Library documentation.
*
* \param mode
* Specifies mode cy_en_syspm_callback_mode_t.
*
* \return
* Returns the status cy_en_syspm_status_t of the operation requested
* by the mode parameter:
* - CY_SYSPM_SUCCESS  - Deep Sleep power mode can be entered.
* - CY_SYSPM_FAIL     - Deep Sleep power mode cannot be entered.
*
*******************************************************************************/
cy_en_syspm_status_t CapSense_1_DeepSleepCallback(
                     cy_stc_syspm_callback_params_t * callbackParams,
                     cy_en_syspm_callback_mode_t mode)
{
    cy_en_syspm_status_t retVal = CY_SYSPM_SUCCESS;
    uint8 temp;

    (void)callbackParams;
    switch(mode)
    {
    case (CY_SYSPM_CHECK_READY):
        temp = CapSense_1_AdcIsBusy();
        if ((CapSense_1_AdcSTATUS_IDLE != temp) &&
            (CapSense_1_AdcSTATUS_OVERFLOW != temp))
        {
            /* Scanning in progress */
            retVal = CY_SYSPM_FAIL;
        }
        break;

    default:
        break;
    }

    return(retVal);
}


#endif /* (CapSense_1_ENABLE == CapSense_1_ADC_STANDALONE_EN) */

#endif /* #if CapSense_1_ADC_EN */


/* [] END OF FILE */
